-- 3DGEN.MS : 3D Mesh and Poly Generator

-- Category    : Geometry
-- Environment : MAX 2.0
-- Description : standard file input, shape & mesh creation
-- Reference   : HELICOID.DAT (mesh data), ROSE?.DAT (shape data)

-- Copyright 1998, EffectWare, Moon-Sun Choi (aeroplus)
-- http://www.chollian.net/~efxware
-- efxware@chollian.net

-- general functions ----------------------------------------------------------

-- read 3D point list from file
-- file : file descriptor

fn read_point file =
(
    local vtx_list = #()
    local pnt = [0.0, 0.0, 0.0]

    while (not eof file) do
    (
        pnt.x = readvalue file
        pnt.y = readvalue file
        pnt.z = readvalue file
        append vtx_list [pnt.x, pnt.y, pnt.z]
    )

    vtx_list
)

-- 3D mesh generator ----------------------------------------------------------

-- vtx_list : vertex list
-- m : mesh M size
-- n : mesh N size

fn draw_mesh vtx_list m n =
(
    local node, i

    if vtx_list == #() then return false
    if m < 2 then return false
    if n < 2 then return false

    node = mesh lengthsegs:(m - 1) widthsegs:(n - 1)

    for i = 1 to m * n do
        setvert node (i) vtx_list[i]
		
    update node
)

-- file_name : data file name

-- DATA file example
-- 2 3   <--- mesh size
-- 0 0 0 <--- index 1, 1
-- 1 0 0            1, 2
-- 2 0 0            1, 3
-- 0 1 0            2, 1
-- 1 1 0            2, 2
-- 2 1 0            2, 3

fn read_data_mesh file_name =
(
    local file
    local vtx_list = #()
    local m, n

    file = openFile file_name
    if file == undefined then false
	
    -- read mesh size
    m = readvalue file
    n = readvalue file

    -- read vertex list
    vtx_list = read_point file
    close file

    -- draw mesh node
    undo on
    (
        draw_mesh vtx_list m n
    )
)

-- 3D mesh generator main program

utility mesh_generator "3D Mesh Generator"
(
    local file_name

    group "About"
    (
        label l1 "3D Mesh Generator"
        label l2 "Version 1.0"
        label l3 "Copyright 1998, EffectWare"
        label l4 "Choi, Moon-Sun (aeroplus)"
        label l5 "www.chollian.net/~efxware"
    )

    group "Parameters"
    (
        button load_data "  Load Data  "
    )

    on load_data pressed do
    (
        file_name = getOpenFileName caption:"Load 3D Mesh Data"

        if file_name != undefined then
            read_data_mesh file_name
    )
)

-- 3D poly generator ----------------------------------------------------------

-- vtx_list : vertex list

fn draw_poly vtx_list =
(
    local shape
    local counts, i

    if vtx_list == #() then return false
	
    shape = splineShape pos:[0, 0, 0]
    addNewSpline shape

    counts = vtx_list.count
    for i = 1 to counts do
    (
        addKnot shape 1 #smooth #curve vtx_list[i]
    )

    updateShape shape
)

-- file_name : data file name

fn read_data_poly file_name =
(
    local file
    local vtx_list = #()

    file = openFile file_name
    if file == undefined then false
	
    -- read vertex list
    vtx_list = read_point file
    close file

    -- draw spline shape node
    undo on
    (
        draw_poly vtx_list
    )
)

-- 3D poly generator main program

utility poly_generator "3D Poly Generator"
(
    local file_name

    group "About"
    (
        label l1 "3D Poly Generator"
        label l2 "Version 1.0"
        label l3 "Copyright 1998, EffectWare"
        label l4 "Choi, Moon-Sun (aeroplus)"
        label l5 "www.chollian.net/~efxware"
    )

    group "Parameters"
    (
        button load_data "  Load Data  "
    )

    on load_data pressed do
    (
        file_name = getOpenFileName caption:"Load 3D Poly Data"
		
        if file_name != undefined then
            read_data_poly file_name
    )
)
